/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_INNO4LIDARSERVICE_INNO4LIDARSERVICE_SKELETON_H
#define JAC_AP_INNO4LIDARSERVICE_INNO4LIDARSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/inno4lidarservice/inno4lidarservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace inno4lidarservice {
namespace skeleton {
namespace events
{
    using LiDAR4PointCloudEvent = ara::com::internal::skeleton::event::EventAdapter<::LidarPointCloudNew>;
    static constexpr ara::com::internal::EntityId Inno4LiDARServiceLiDAR4PointCloudEventId = 55541U; //LiDAR4PointCloudEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class Inno4LiDARServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultLiDAR4PointCloudEvent = LiDAR4PointCloudEvent.Initialize();
        ThrowError(resultLiDAR4PointCloudEvent);
    }

    Inno4LiDARServiceSkeleton& operator=(const Inno4LiDARServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceId, mode)),
              LiDAR4PointCloudEvent(ptr->GetSkeleton(), events::Inno4LiDARServiceLiDAR4PointCloudEventId, ::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceSpec, mode)),
              LiDAR4PointCloudEvent(ptr->GetSkeleton(), events::Inno4LiDARServiceLiDAR4PointCloudEventId, ::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceContainer, mode)),
              LiDAR4PointCloudEvent(ptr->GetSkeleton(), events::Inno4LiDARServiceLiDAR4PointCloudEventId, ::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              LiDAR4PointCloudEvent(std::move(other.LiDAR4PointCloudEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                LiDAR4PointCloudEvent = std::move(other.LiDAR4PointCloudEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::LiDAR4PointCloudEvent GetLiDAR4PointCloudEvent()
        {
            return std::move(LiDAR4PointCloudEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = LiDAR4PointCloudEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::LiDAR4PointCloudEvent LiDAR4PointCloudEvent;
    };
    explicit Inno4LiDARServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceId, mode)),
          LiDAR4PointCloudEvent(skeletonAdapter->GetSkeleton(), events::Inno4LiDARServiceLiDAR4PointCloudEventId, ::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit Inno4LiDARServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceSpec, mode)),
          LiDAR4PointCloudEvent(skeletonAdapter->GetSkeleton(), events::Inno4LiDARServiceLiDAR4PointCloudEventId, ::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit Inno4LiDARServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceContainer, mode)),
          LiDAR4PointCloudEvent(skeletonAdapter->GetSkeleton(), events::Inno4LiDARServiceLiDAR4PointCloudEventId, ::jac::ap::inno4lidarservice::Inno4LiDARService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    Inno4LiDARServiceSkeleton(const Inno4LiDARServiceSkeleton&) = delete;

    Inno4LiDARServiceSkeleton(Inno4LiDARServiceSkeleton&&) = default;
    Inno4LiDARServiceSkeleton& operator=(Inno4LiDARServiceSkeleton&&) = default;
    Inno4LiDARServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          LiDAR4PointCloudEvent(token.GetLiDAR4PointCloudEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~Inno4LiDARServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&Inno4LiDARServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::LiDAR4PointCloudEvent LiDAR4PointCloudEvent;
};
} // namespace skeleton
} // namespace inno4lidarservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_INNO4LIDARSERVICE_INNO4LIDARSERVICE_SKELETON_H
